// Copyright 2013-2014, SAS Institute Inc., Cary, NC, USA.  All Rights Reserved.

package com.sas.deployment
import groovy.transform.InheritConstructors

import com.sas.gate.CodeAccessor
import com.sas.gate.api.PipeCode
import com.sas.gate.api.template.*
import org.codehaus.groovy.runtime.StackTraceUtils

/**
   <overview>Deployment code for SAS Enterprise Case Management (12byte)</overview>
   <p>
      This code configures the deployment of 12byte using what corresponds to
      the following Ant targets:
      <ul class='nodot'>
         <li>configure</li>
         <li>buildWebapps</li>
         <li>deployWebapps</li>
         <li>loadContent</li>
         <li>unconfigure</li>
      </ul>
   </p>

   @since      14w14
   @version    15w04
   @author     Bruce Bryan
   @see        #SOURCE#
*/

@InheritConstructors
class CASEMGMTMID extends CodeAccessor
{
   // ===================================================== GLOBAL

   /**
      Identifier

      @param id <type>String</type>: The 12byte name
   */
   String id = 'CASEMGMTMID'

   /**
      Callback for hook: <hook>properties &gt; init</hook>

      @param t__properties_get   <type>ArrayList&lt;String&gt;</type>: get object properties on each dependson property

   <ul class='nodot'>
      <li>levconfig</li>
      <li>workspacesrv</li>
   </ul>
   */
   def properties_init =
   {
      pass.with
      {
          // Simple Strings
         _releaseNumber                      = api.'casemgmtmid.release.number'
         _offeringLegalName                  = api.'casemgmtmid.offering.legal.name'
         _normalizedOfferingLegalName        = api.'casemgmtmid.normalized.offering.name'
         _normalizedShortLegalName           = api.'casemgmtmid.normalized.short.name'
         _rootAppName                        = 'sas.solutions.casemgmt'
         _shortLegalName                     = api.'casemgmtmid.short.legal.name'
         _softwareComponentName              = api.'casemgmtmid.software.component.name'
         _shortName                          = 'EntCaseManagement'

         // Config Paths
         _sasConfigDir                       = api.'config.lev.web.applications.dir'
         _configDir                          = canonical("$_sasConfigDir/$_normalizedOfferingLegalName")
         _configInstallDir                   = canonical("$_configDir/install")
         _configMetadataDir                  = canonical("$_configInstallDir/metadata")

         // Install Paths (Dirs and Files)
         _installDir                         = api.'casemgmtmid.install.dir'
         _installConfigDir                   = api.'casemgmtmid.install.config.dir'
         _installMetadataDir                 = canonical("$_installConfigDir/Deployment/Metadata")
         _installSasmiscDir                  = api.'casemgmtmvac.install.sasmisc.dir'

         // Other
         _contextName                        = "SAS$_shortName"
         _davSubFolder                       = "$_shortName$_releaseNumber"
         _webappsrvHost                      = api.'casemgmtmid.webappsrv.host'
         _webappsrvPort                      = api.'casemgmtmid.webappsrv.port'.trim()
         _webappsrvAutoPort = (_webappsrvPort in ['', '80', '443']) ? '' : ":$_webappsrvPort"
         _webappsrvProtocol                  = api.'casemgmtmid.webappsrv.protocol'
         _webappsrvUrl                       = api.'casemgmtmid.webappsrv.url'
         _ecmUrl                             = "$_webappsrvUrl$_contextName"
         _dbmsAuthdomainName                 = api.'casemgmtmid.dbms.authdomain'
         _dbmsEngine                         = api.'casemgmtmid.dbms.engine'
         _databaseHost                       = api.'casemgmtmid.dbms.host'
         _databasePort                       = api.'casemgmtmid.dbms.port'
         _databaseName                       = api.'casemgmtmid.dbms.database.name'

         // Template APIs
         t__properties_get =
         [
            'casemgmtmid.dependson.webappsrv.fqid',
            'casemgmtmid.dependson.webinfpltfm.fqid',
            'casemgmtmid.dependson.casemgmtmvac.fqid',
            'casemgmtmid.dependson.levconfig.fqid',
         ]
      }
   }

   /**
      Callback for hook: <hook>properties &gt; post</hook>

      @param filters <type>Map&lt;String&gt;</type>: tokens for string substitution
      @param t__setobjectproperties    <type>ArrayList&lt;String&gt;</type>: Properties that will be stored in the configured objects text store
      @param t__createdirectories <type>Map&lt;String&gt;</type>: directories to be created in configure target<br>
            <indent><key>prefix</key>- <type>String</type>: base directory to create<br>
            <key>entries</key>- <type>ArrayList&lt;String&gt;</type>: directories and subdirectories to be created<br></indent>
      <pre>
   SAS_CONFIG/Lev1/Web/Applications/SASCustomerDueDiligence6.1
   `-- install
    `-- metadata</pre>
      @param t__copyfiles         <type>ArrayList&lt;Map&gt;</type>: files to copy and conditions for the copy<br>
            <indent><key>src</key>- <type>String</type>: Source directory to copy from<br>
            <key>include</key>- <type>String</type>: Regex pattern of file(s) to copy<br>
            <key>dst</key>- <type>String</type>: Destination directory for files to be copied to<br>
            <key>strip</key>- <type>String</type> <kind>optional</kind>: Remove extension form file name i.e., remove .orig extension<br>
            <key>tokens</key>- <type>String</type> <kind>optional</kind>: filterset to use for token replacement<br></indent>
      @param t__loadmetadata_createfolders   -
      @param t__importpackages_createfolders   -
      @param t__loadmetadata_load   <type>String</type>: Fully qualified path to the token substituted appxml file
      @param t__sasjobs_run      -
   */
   def properties_post =
   {
      String appServerName          = api.'oma.appserver.servercontext.name'
      String cfgLevWebLogsDir       = api.'config.lev.web.logs.dir'
      String classIdentifier        = api.'casemgmtmid.configobj.classid'
      String configobjName          = api.'casemgmtmid.configobj.name'
      String contextRoot            = "/SAS$pass._shortName"
      String davFolder              = pass._contextName
      String displayName            = "SAS $pass._shortLegalName $pass._releaseNumber"
      String majorNumber            = api.'casemgmtmid.major.number'
      String minorNumber            = api.'casemgmtmid.minor.number'
      String repositoryName         = api.'oma.repository.foundation.name'
      String schemaName             = api.'casemgmtmid.dbms.schema'
      String trustedUserName        = api.'oma.person.trustusr.person.name'
      Map serverLogDirs             = api.serverLogDirs()
      Boolean searchEnabled         = getProperty('casemgmtmid.search.enabled', '0') { toBoolean(it) }
      String searchUrl              = searchEnabled ? api.'casemgmtmid.search.url' : ''
      String searchCollection       = api.'casemgmtmid.search.collection'
      String searchgetProtocol      = api.'casemgmtmid.search.get.protocol'
      String databaseName           = (api.'casemgmtmid.dbms.engine' == 'oracle') ? '' : "${api.'casemgmtmvac.dbms.database.name'}"
      String databaseSid            = (api.'casemgmtmid.dbms.engine' == 'oracle') ? "${api.'casemgmtmvac.dbms.oracle.sid'}" : ''

      // Properties
      api.createProperty('casemgmtmid.log.dir.html', serverLogDirs.html)
      api.createProperty('casemgmtmid.display.name', displayName)
      api.createProperty('casemgmtmid.dav.subfolder.release', pass._davSubFolder)
      if (databaseName) api.createProperty('casemgmtmid.dbms.database.name', databaseName)
      if (databaseSid) api.createProperty('casemgmtmid.dbms.oracle.sid', databaseSid)

      if (t__debug){
         assert classIdentifier == 'casemgmtmid' : "Class identifier does not match casemgmtmid"
         assert majorNumber == '6' : "Release major number does not equal 6"
         assert minorNumber == '3' : "Release minor number does not equal 3"
      }

      pass.with
      {
         _serverSoftwareComponentName           = api.'casemgmtmvac.software.component.name'
         _samplesEnabled                        = getProperty('casemgmtmid.samples.enabled', '0') { toBoolean(it) }
         _filters =
         [
            'casemgmtmid.configobj.classid'        : classIdentifier,
            'casemgmtmid.configobj.name'           : configobjName,
            'casemgmtmid.dav.folder'               : davFolder,
            'casemgmtmid.dav.subfolder.release'    : _davSubFolder,
            'casemgmtmid.db.schema.name'           : schemaName,
            'casemgmtmid.host'                     : _webappsrvHost,
            'casemgmtmid.port'                     : _webappsrvPort,
            'casemgmtmid.auto.port'                : _webappsrvAutoPort,
            'casemgmtmid.protocol'                 : _webappsrvProtocol,
            'casemgmtmid.search.enabled'           : searchEnabled.toString(),
            'casemgmtmid.search.url'               : searchUrl,
            'casemgmtmid.search.collection'        : searchCollection,
            'casemgmtmid.search.get.protocol'      : searchgetProtocol,
            'casemgmtmvac.software.component.name' : _serverSoftwareComponentName,
            'config.lev.web.logs.dir'              : cfgLevWebLogsDir,
            'context.name'                         : _contextName,
            'jms.jndi.prefix'                      : '',
            'log.level.default'                    : 'WARN',
            'major.number'                         : majorNumber,
            'minor.number'                         : minorNumber,
            'oma.appserver.servercontext.name'     : appServerName,
            'oma.person.trustusr.person.name'      : trustedUserName,
            'oma.repository.foundation.name'       : repositoryName,
            'webapp.casemgmt.contextroot'          : contextRoot,
            'webapp.casemgmt.display.name'         : displayName,
            'webapp.casemgmt.installdir.name'      : _rootAppName,
            'webapp.casemgmtmid.configobj.classid' : classIdentifier,
            'webapp.casemgmtmid.contextroot'       : contextRoot,
            'webapp.casemgmtmid.legal.name'        : _offeringLegalName,
            'webapp.casemgmtmid.legal.short.name'  : _shortLegalName,
            'webapp.casemgmtmid.release.number'    : _releaseNumber,
            'webapp.casemgmtmid.short.name'        : _shortName,
            'webapp.casemgmtmid.software.component.name' : _softwareComponentName,
         ]

         t__setobjectproperties =
         [
            properties:
            [
               'casemgmtmid.context.name',
               'casemgmtmid.dbms.authdomain',
               'casemgmtmid.dbms.engine',
               'casemgmtmid.dbms.host',
               'casemgmtmid.dbms.port',
               'casemgmtmid.dbms.schema',
               'casemgmtmid.dbms.type',
               'casemgmtmid.dbms.userid',
               'casemgmtmid.release.number',
               'casemgmtmid.software.component.name'
            ]
         ]
         switch(_dbmsEngine)
         {
         case 'db2' :
            t__setobjectproperties.properties +=
               [
               'casemgmtmid.dbms.database.name',
               'casemgmtmid.dbms.db2_license'
               ]
            break
         case 'oracle' :
            t__setobjectproperties.properties +=
            [
               'casemgmtmid.dbms.oracle.net.service.name',
               'casemgmtmid.dbms.oracle.sid'
            ]
            break
         default :
            t__setobjectproperties.properties +=
            [
                  'casemgmtmid.dbms.database.name'
            ]
            break
         }
      }
   }

   def properties_get =
   {
      //api.stop(properties:true)
   }

   def properties_assert =
   {
   println '########## in debug mode'
      pass.with {
         assert _releaseNumber == "6.3" : "Release number does not match 6.3"
         assert _offeringLegalName == 'SAS Enterprise Case Management' : "Offering legal name does not match SAS Enterprise Case Management"
         assert _shortLegalName == 'Ent Case Mgmt Mid-Tier' : "Short Legal name does not match Ent Case Mgmt Mid-Tier"
         assert _softwareComponentName == 'Ent Case Mgmt Mid-Tier 6.3' : "Software component name does not match Ent Case Mgmt Mid-Tier 6.3"
         assert _configDir =~ "SASEnterpriseCaseManagement" : "Config dir does not contain SASEnterpriseCaseManagement"
         assert _configInstallDir =~ "install" : "Config install dir does not contain install"
         assert _configMetadataDir =~ "metadata" : "Config metadata dir does not contain metadata"
      }
   }

   // ===================================================== TARGET: configure

   /**
      Target: configure

      @see com.sas.gate.api.template.ConfigureTemplate
   */
   def configure()
   {
      //api.stop()

      pass.with {
         t__debug = getProperty('wizard.debug.mode', '0') { toBoolean(it) }
      }

      ConfigureTemplate t = new ConfigureTemplate(this)

      t['migration > init']                  .add(configure__migration_init)
      t['properties > init']                 .add(properties_init)
      t['properties > post']                 .add(properties_post)
      t['properties > post']                 .add(configure_migrate_properties_post).when('t__migrate')
      t['properties > post']                 .add(properties_assert).when('t__debug')
      t['init']                              .add(configure__init)
      t['loadmetadata > migration > init']   .add(configure__loadmetadata_migration_init).when('t__migrate')
      t['docparts > init']                   .add(configure__docparts_init).when('!t__migrate')
      t['docparts > migration']              .add(configure__docparts_migration).when('t__migrate')

      t.execute()

      //api.stop(properties:true)
   }

   def configure__migration_init =
   {
      String maintenanceFrom  = getProperty('casemgmtmid.maintenance.from.version', '')
      String fromVersion      = api.baseVersion(maintenanceFrom)
      String releaseNumber    = api.'casemgmtmid.release.number'

      String configDir = canonical("${api.'config.lev.web.applications.dir'}/${api.'casemgmtmid.normalized.offering.name'}")
      api.createProperty('casemgmtmid.config.web.dir', configDir)
      api.createProperty('casemgmtmid.logs.dir', api.'config.lev.web.logs.dir')

      pass.with {
         _updateMetadata =
         t__migrate = getProperty('casemgmtmid.migration.is_enabled', '0') { toBoolean(it) }
         if (t__migrate)
         {
            if ( maintenanceFrom != releaseNumber )
            {
               t__searchandcall =
               [
                  actions: [ __template.t__deletemetadata ],
                  fqids:
                  [
                     [
                        type:    'Tree',
                        search:  "@Name = 'Ent Case Mgmt Mid-Tier $fromVersion'"
                     ]
                  ]
               ]
            }
         }
      }
   }

   def configure_migrate_properties_post =
   {
      pass.with {
         t__setobjectproperties +=
         [
            existingproperties: 'replace'
         ]
      }
   }

   def configure__init =
   {
      String datasourceName                     = api.'casemgmtmid.dbms.datasource.name'
      String jndiName                           = "sas/jdbc/$datasourceName"
      String validationQuery                    = api.'casemgmtmid.dbms.test.connection'
      String serverName                         = api.serverName()

      api.createWeb()

      pass.with {
         t__createdirectories =
         [
            prefix: _configDir,
            entries:
            [
               'install/metadata',
            ]
         ]

         t__copyfiles =
         [
            [  src:        _installMetadataDir,
               include:    ~/.*\.orig/,
               dst:        _configMetadataDir,
               strip:      '.orig',
               tokens:     _filters
            ],
            [  src:        _installMetadataDir,
               include:    ~/.*\.locprop/,
               exclude:    ~/.*\.orig/,
               dst:        _configMetadataDir,
            ],
         ]

         t__loadmetadata_load = canonical("$_configMetadataDir/casemgmtmid.appxml")

         t__datasource_create =
         [
            adminUser:        api.'casemgmtmid.dbms.userid',
            adminPW:          api.decodeSASstring(api.'casemgmtmid.dbms.passwd'),
            classPath:        api.'casemgmtmid.dbms.jar',
            connectionURL:    api.'casemgmtmid.dbms.url',
            datasourceName:   datasourceName,
            driver:           api.'casemgmtmid.dbms.driver_class',
            jndiName:         jndiName,
            servername:       serverName,
            validationQuery:  validationQuery,
         ]

         t__datasource_connection =
         [
            connectionFactoryName:  'ECMQueueConnectionFactory',
            servername:             serverName,
         ]

         t__datasource_queue =
         [
            queueName:              'AlertQueue',
            servername:             serverName,
         ]
      }
   }

   def configure__loadmetadata_migration_init = 
   {
      String applicationSubFolder = api.'casemgmtmid.application.sub.folder'

      pass.with {
         t__loadmetadata_migration_update =
         [
            file: canonical("$_configMetadataDir/casemgmtmid.appxml"),
            folder: applicationSubFolder,
            applicationname: _softwareComponentName
         ]
      }
   }

   /**
      Callback for hook: <hook>properties &gt; post</hook>

      @param t__docparts   <type>ArrayList&lt;Map&gt;</type>: tokens in Instructions.html to replace and properties to replace them with
            <indent><key>replace, prepend, append</key>- <type>String</type>: Type of replacement to perform<br>
            <indent><key>token</key>- <type>String</type>: Token in Instructions.html to replace<br>
            <key>value</key>- <type>String</type>: Property to replace token with</indent></indent>

   */
   def configure__docparts_init =
   {
      pass.with
      {
         t__docparts =
         [
         actions:
            [
               [replace:[
                  token:'instructions.casemgmtmid.head',
                  value:api.'instructions_casemgmtmid_head.txt']],
               [replace:[
                  token:'instructions.casemgmtmid.details.table.head',
                  value:api.'instructions_casemgmtmid_details_table_head.txt']],
               [prepend:[
                  token:'instructions.casemgmtmid.details.table.item',
                  value:api.'instructions_casemgmtmid_details_doc.txt']],
               [replace:[
                  token:'instructions.casemgmtmid.details.table.tail',
                  value:api.'instructions_casemgmtmid_details_table_tail.txt']],
            ]
         ]
      }
   }

   def configure__docparts_migration =
   {
      pass.with {
         t__docparts =
         [
         actions:
            [
               [replace:[
                  token:'instructions.casemgmtmid.head',
                  value:api.'instructions_casemgmtmid_head.txt']],
               [replace:[
                  token:'instructions.casemgmtmid.details.table.head',
                  value: api.'migrateconfig_report.txt']],
               [prepend:[
                  token:'instructions.casemgmtmid.details.table.item',
                  value:api.'instructions_casemgmtmid_details_doc.txt']],
               [replace:[
                  token:'instructions.casemgmtmid.details.table.tail',
                  value:api.'instructions_casemgmtmid_details_table_tail.txt']],
            ]
         ]
      }
   }

   def sasjobs_createuser =
   {
      api.createUserAddToGroups(pass.t__sasjobs_run)
   }

   def sasjobs_removeuser =
   {
      api.removeUserFromGroupsAndDelete(pass.t__sasjobs_run)
   }

   // ===================================================== TARGET: buildWebapps

   /**
      Target: buildWebapps

      @see com.sas.gate.api.template.BuildWebappsTemplate
   */
   def buildWebapps()
   {
      //api.stop()

      BuildWebappsTemplate t = new BuildWebappsTemplate(this)

      t['properties > init']        .add(properties_init)
      t['properties > post']        .add(properties_post)
      t['stage > init']             .add(stage_init)
      t['docparts > init']          .add(buildWebapps__docparts_init)

      t.execute()

      //api.stop()
   }

   def stage_init =
   {
      String configurableDir                    = canonical("$pass._installDir/Configurable")
      String commonLogDir                       = api.'config.lev.web.common.log_config.dir'
      String picklistsDir                       = canonical("$pass._installDir/Picklists")
      String picklistFile                       = canonical("$picklistsDir/wars/$pass._rootAppName/picklist")
      String staticDir                          = canonical("$pass._installDir/Static")
      String configurableWarsDir                = canonical("$configurableDir/wars/$pass._rootAppName")
      String configurableWarsWEBINFDir                = canonical("$configurableDir/wars/$pass._rootAppName/WEB-INF")
      String configurableClassesDir             = canonical("$configurableDir/wars/$pass._rootAppName/WEB-INF/classes")
      String configurableEarsDir                = canonical("$configurableDir/ears/$pass._rootAppName")
      String configurableAppsDir                = canonical("$configurableDir/apps/$pass._contextName")
      String staticWarsDir                      = canonical("$staticDir/wars/$pass._rootAppName")
      String staticEarsDir                      = canonical("$staticDir/ears/$pass._rootAppName")
      String tempDir                            = api.'config.lev.web.temp.dir'
      String tempEarDir                         = canonical("$tempDir/$pass._rootAppName")
      String tempWarDir                         = canonical("$tempEarDir/$pass._rootAppName")
      String description                        = api.'casemgmtmid.description'
      String midAppHost                         = api.'vfabrcersvrc.host'

      String webinfdsvrcFQID = api.getmetadatavalues([
         type:    'ServerComponent',
         search:  "@Name='Web Infrastructure Platform Data Server 9.4']",
      ])
      api.getObjectProperties(webinfdsvrcFQID)
      String authenticator                      = api.'iomsrv.webinfdsvrc.host'

      pass.with
      {
         _filters +=
         [
            'webapp.casemgmt.description'          : description,
            'authenticator'                        : authenticator,
            'ecmHost'                              : midAppHost,
         ]

         t__stage_extract =
         [
            retrieveFromVJR:
            [
               [
                  packageName: _rootAppName,
                  picklist:    picklistFile,
                  dst:         tempWarDir,
               ],
            ]
         ]

         t__createdirectories =
         [
            prefix: tempDir,
            entries:
            [
               "$_rootAppName/META-INF" as String,
               "$_rootAppName/$_rootAppName/WEB-INF" as String,
            ]
         ]

         t__copyfiles =
         [
            [  src:        configurableEarsDir,
               recurse:    true,
               include:    ~/.*\.orig/,
               dst:        "$tempEarDir/META-INF",
               strip:      '.orig',
               tokens:     _filters
            ],
            [  src:        configurableAppsDir,
               recurse:    true,
               include:    ~/.*\.orig/,
               dst:        commonLogDir,
               strip:      '.orig',
               tokens:     _filters
            ],
            [  src:        configurableWarsWEBINFDir,
               include:    ~/.*\.orig/,
               dst:        "$tempWarDir/WEB-INF",
               strip:      '.orig',
               tokens:     _filters
            ],
            [  src:        configurableClassesDir,
               include:    ~/.*\.orig/,
               dst:        "$tempWarDir/WEB-INF/classes",
               strip:      '.orig',
               tokens:     _filters
            ],
         ]

         t__buildarchive =
         [
            buildArchive:
            [
               [
                  appNames  : _rootAppName,
               ]
            ]
         ]
      }
   }

   /**
      Callback for hook: <hook>docparts &gt; init</hook>

      @param t__docparts   <type>ArrayList&lt;Map&gt;</type>: tokens in Instructions.html to replace and properties to replace them with
            <indent><key>replace, prepend, append</key>- <type>String</type>: Type of replacement to perform<br>
            <indent><key>token</key>- <type>String</type>: Token in Instructions.html to replace<br>
            <key>value
            :let @/="\\<</key>- <type>String</type>: Property to replace token with</indent></indent>

            \\>"
            </key>- <type>String</type>: Property to replace token with</indent></indent>

   */
   def buildWebapps__docparts_init =
   {
      pass.with
      {
         t__docparts =
         [
            actions:
            [
               [prepend:[
                  token:'instructions.casemgmtmid.details.table.item',
                  value:api.'instructions_casemgmtmid_details_URL.txt']],
               [prepend:[
                  token:'instructions.casemgmtmid.details.table.item',
                  value:api.'instructions_casemgmtmid_details_logging.txt']],
            ]
         ]

         Boolean inUIP = getProperty('casemgmtmid.maintenance.from.version', '0') { toBoolean(it) }
         if (!(t__migrate || inUIP))
         {
            t__docparts.actions +=
                  [prepend:[
                     token:'instructions.casemgmtmid.details.table.item',
                     value:api.'instructions_casemgmtmid_details_validate.txt']
                  ]
         }
      }
   }
   // ===================================================== TARGET: deployWebapps

   /**
      Target: deployWebapps

      @see com.sas.gate.api.template.DeployWebappsTemplate
   */
   def deployWebapps()
   {
      //api.stop()

      DeployWebappsTemplate t = new DeployWebappsTemplate(this)

      t['properties > init']        .add(properties_init)
      t['properties > post']        .add(properties_post)
      t['deploywebapps > init']     .add(deploywebapps_init)

      t.execute()

      //api.stop()
   }

   def deploywebapps_init =
   {
      String appName                = "SAS$pass._normalizedShortLegalName$pass._releaseNumber"
      String stagingDir             = canonical("${api.'config.lev.web.staging.dir'}/${pass._rootAppName}.ear")

      pass.with {
         t__deploywebapps =
         [
            deploy:
            [
               [
                  pathtoear:  stagingDir,
                  appname:    appName,
               ],
            ]
         ]
      }
   }

   // ===================================================== TARGET: loadContent

   /**
      Target: loadContent

      @see com.sas.gate.api.template.LoadContentTemplate
   */
   def loadContent()
   {
      //api.stop()

      LoadContentTemplate t = new LoadContentTemplate(this)

      t['properties > init']        .add(properties_init)
      t['properties > post']        .add(properties_post)

      t['loadcontent > init']       .add(loadcontent_init)
      t['loadattachments > init']   .add(loadattachments_init)

      t['loadworkflows > init']     .add(loadworkflows_init)

      t['createwebservice > init']  .add(createwebservice_init)
      t['docparts > init']          .add(loadContent__docparts_init)

      t.execute()

      //api.stop()
   }

   /**
      Callback for hook: <hook>loadcontent &gt; init</hook>

      @param concatenate   Concatenates the custom.properties files from CDD, ECM and AML if included in the plan.
            <indent><key>src</key>- <type>ArrayList&lt;Map&gt;</type>: Array of maps containing the source directories of the files that are to be concatenated<br>
               <indent><key>dir</key>- <type>Map</type>: Map of an array of directories containing files to be concatenated</indent>
            <key>dst</key>- <type>String</type>: Destination where contatinated files will be left including filename, may include regex expression.<br>
            <key>include</key>- <type>String</type>: regular expression used to select files to be concatenated</indent>
      @param menuInject <type>ArrayList&lt;Map&gt;</type>: Inserts the CDD MainNavigationMenu.xml file into the appropriate location in the ECM provided MainNavigationMenu.xml file.
      @param t__loadcontent <type>ArrayList&lt;Map&gt;</type>: Uploads the prepared files to the SAS Content Server.
            <indent><key>failifmissing</key>- <type>Boolean</type>: Fail if files are missing<br>
               <key>content</key>- <type>ArrayList&lt;Map&gt;</type>: Array of content to be uploaded to DAV<br>
            <indent><key>urlSuffix</key>- <type>String</type> <kind>optional</kind>: Suffix to be aplied to the DAV URL where content is to be uploaded to.<br>
            <key>contentLocation</key>- <type>String</type>: Fully qualified path to the Directory where content to be uploaded resides.</indent></indent>
   */
   def loadcontent_init =
   {
      // Source directories
      String contentSrc             = canonical("$pass._installConfigDir/Deployment/Content")

      String installDeployDir       = canonical("$pass._installDir/deploy")

      String requiredSrc            = canonical("$installDeployDir/required")
      String templateSrc            = canonical("$requiredSrc/Templates")
      String uiDefinitionsSrc       = canonical("$requiredSrc/UIDefinitions")
      String menusSrc               = canonical("$requiredSrc/Menus")

      String sampleSrc              = canonical("$installDeployDir/sample")
      String sampleCPSrc            = canonical("$sampleSrc/CustomProperties")
      String sampleMenusSrc         = canonical("$sampleSrc/Menus")
      String sampleSearchConfigSrc  = canonical("$sampleSrc/SearchConfig")
      String sampleUIDefinitionsSrc = canonical("$sampleSrc/UIDefinitions")
      String sampleWorkflowsSrc     = canonical("$sampleSrc/Workflows")
      String sampleReportConfigSrc  = canonical("$sampleSrc/ReportConfig")

      // Destination directories
      String contentDst             = canonical("$pass._configInstallDir/content")
      String contentsDst            = canonical("$pass._configInstallDir/contents")

      String sampleSearchConfigDst  = canonical("$pass._configInstallDir/SearchConfig")
      String sampleWorkflowsDst     = canonical("$pass._configInstallDir/Workflows")
      String sampleReportConfigDst  = canonical("$pass._configInstallDir/ReportConfig")

      String cpSuffix            = "/Products/SAS$pass._shortName/$pass._davSubFolder/CustomProperties"
      String menuSuffix          = "/Products/SAS$pass._shortName/$pass._davSubFolder/Menus"
      String searchConfigSuffix  = "/Products/SAS$pass._shortName/$pass._davSubFolder/SearchConfig"
      String uiSuffix            = "/Products/SAS$pass._shortName/$pass._davSubFolder/UIDefinition"
      String reportConfigSuffix  = "/Products/SAS$pass._shortName/$pass._davSubFolder/ReportConfig"

      pass.with
      {
         if (api.'wizard.type'=='sdw') t__initializemanualcontent_load = true
         else t__initializemanualcontent_update = true

         t__copyfiles =
         [
            [  src:        contentSrc,
               include:    ~/.*\.orig/,
               dst:        contentDst,
               strip:      '.orig',
               tokens:     _filters
            ],
            [  recurse:    true,
               src:        templateSrc,
               base:       requiredSrc,
               include:    ~/.*\.st/,
               dst:        contentsDst
            ],
            [
               enabled:    _samplesEnabled,
               src:        sampleSearchConfigSrc,
               include:    ~/.*\.orig/,
               dst:        sampleSearchConfigDst,
               strip:      '.orig',
               tokens:     _filters
            ],
            [
               enabled:    _samplesEnabled,
               src:        sampleWorkflowsSrc,
               include:    ~/.*\.xml/,
               dst:        sampleWorkflowsDst
            ],
            [
               enabled:    _samplesEnabled,
               src:        sampleReportConfigSrc,
               include:    ~/.*\.xml/,
               dst:        sampleReportConfigDst
            ]
         ]

         t__loadcontent_upload =
         [
            failifmissing: true,
            content:
            [
               [
                  contentLocation:  contentDst,
               ],
               [
                  contentLocation:  contentsDst,
               ],
               [
                  contentLocation:  uiDefinitionsSrc,
                  urlSuffix:        uiSuffix,
               ],
               [
                  contentLocation:  menusSrc,
                  urlSuffix:        menuSuffix,
               ],
               [
                  enabled:          _samplesEnabled,
                  contentLocation:  sampleCPSrc,
                  urlSuffix:        cpSuffix,
               ],
               [
                  enabled:          _samplesEnabled,
                  contentLocation:  sampleMenusSrc,
                  urlSuffix:        menuSuffix,
               ],
               [
                  enabled:          _samplesEnabled,
                  contentLocation:  sampleSearchConfigDst,
                  urlSuffix:        searchConfigSuffix,
               ],
               [
                  enabled:          _samplesEnabled,
                  contentLocation:  sampleUIDefinitionsSrc,
                  urlSuffix:        uiSuffix,
               ],
            ]
         ]
      }
   }

   /**
      Callback for hook: <hook>loadattachments &gt; init</hook>

      @param t__loadattachments <type>Map</type>: Uploads the prepared files to the SAS WIP Attachment Service
            <indent><key>password</key>- <type>String</type>: Encoded password generally sasTrust<br>
            <key>urlPrefix</key>- <type>String</type>: Http address of the SASWIPClientAccess/rest/content/<br>
            <key>paths</key>- <type>ArrayList&lt;Map&gt;</type>: Array of maps containing the source that will be uploaded to the attachment service<br>
               <indent><key>dir</key>- <type>String</type>: Fully qualified path to the directory where the files are located that are to be uploaded<br>
               <key>include</key>- <type>String</type>: Regular expression used to select files to be concatenated<br>
               <key>urlSuffix</key>- <type>String</type>: String that will be appended to the urlPrefix to upload the files to the correct location</indent><br>
            </indent>
   */
   def loadattachments_init =
   {
      String password         = api.'oma.person.trustusr.login.passwd'
      String protocol         = api.'iomsrv.httpserver.protocol.type'
      String host             = api.'iomsrv.httpserver.host'
      String port             = api.'iomsrv.httpserver.port'
      String urlPrefix        = "$protocol://$host:$port/SASWIPClientAccess/rest/content"

      String installDeployDir       = canonical("$pass._installDir/deploy")
      String sampleCPSrc            = canonical("$installDeployDir/sample/CustomProperties")
      String uiDefinitionsSrc       = canonical("$installDeployDir/required/UIDefinitions")
      String sampleUIDefinitionsSrc = canonical("$installDeployDir/sample/UIDefinitions")
      String sampleSearchConfigDst  = canonical("$pass._configInstallDir/SearchConfig")
      String sampleReportConfigDst  = canonical("$pass._configInstallDir/ReportConfig")

      pass.with
      {
         t__loadattachments =
         [
            password:   password,
            urlPrefix:  urlPrefix,
            paths:
            [
               [
                  enabled:    _samplesEnabled,
                  dir:        sampleCPSrc,
                  include:    ~/.*\.properties/,
                  urlSuffix:  "545004/CustomProperties/attachments",
               ],
               [
                  dir:        uiDefinitionsSrc,
                  include:    ~/.*\.xml/,
                  urlSuffix:  "545003/UIDefinition/attachments",
               ],
               [
                  enabled:    _samplesEnabled,
                  dir:        sampleUIDefinitionsSrc,
                  include:    ~/.*\.xml/,
                  urlSuffix:  "545003/UIDefinition/attachments",
               ],
               [
                  enabled:    _samplesEnabled,
                  dir:        sampleSearchConfigDst,
                  include:    ~/.*\.xml/,
                  urlSuffix:  "545100/SearchConfig/attachments",
               ],
               [
                  enabled:    _samplesEnabled,
                  dir:        sampleReportConfigDst,
                  include:    ~/.*\.xml/,
                  urlSuffix:  "545300/ReportConfig/attachments",
               ],
            ]
         ]
      }
   }

   def loadworkflows_init =
   {
      // all of this conditional on samples?
      String workFlowDir  = canonical("$pass._installDir/deploy/sample/Workflows")
      ArrayList<String> workflowFiles =
      [
         'CaseManagementFinancialFraud.xml',
         'CaseManagementGeneric.xml',
         'FINCENReport.xml'
      ]

      pass.with
      {
         t__loadworkflows = [enabled: _samplesEnabled, workflows: []]

         workflowFiles.each { file ->
            String workflowFile = canonical("$workFlowDir/$file")
            t__loadworkflows.workflows +=
            [
               srcfile: workflowFile,
               dst:     canonical("$_configInstallDir/workflow")
            ]
         }
      }
   }

   def createwebservice_init =
   {
      String stpPrefix                          = "/System/Applications/$pass._offeringLegalName/$pass._serverSoftwareComponentName/Application SAS Code"
      String picklistsDir                       = canonical("$pass._installDir/Picklists")

      pass.with {
         t__createwebservice =
         [
            services:
            [
               [
                  serviceName       : 'ECMSocialNetworkAnalysis',
                  storedProcesses   :
                  [
                     "$stpPrefix/getSocialNetwork",
                     "$stpPrefix/growSocialNetworkNode",
                     "$stpPrefix/getSocialNetworkNodeDetails"
                  ],
                  namespace         : 'http://sas.com/sso/fraud/sna',
                  publishMetadata   : true,
                  overwrite         : true,
                  picklist          : "$picklistsDir/makeWebService/picklist",
               ],
            ]
         ]
      }
   }

   /**
      Callback for hook: <hook>docparts &gt; init</hook>

      @param t__docparts   <type>ArrayList&lt;Map&gt;</type>: tokens in Instructions.html to replace and properties to replace them with
            <indent><key>replace, prepend, append</key>- <type>String</type>: Type of replacement to perform<br>
            <indent><key>token</key>- <type>String</type>: Token in Instructions.html to replace<br>
            <key>value
            :let @/="\\<</key>- <type>String</type>: Property to replace token with</indent></indent>

            \\>"
            </key>- <type>String</type>: Property to replace token with</indent></indent>

   */
   def loadContent__docparts_init =
   {
      pass.with
      {
         if (!pass.hasVariable('t__docparts')) t__docparts = [:]
         if (!t__docparts.containsKey('actions')) t__docparts.actions = []
         t__docparts.actions +=
         [
            [prepend:[
               token:'instructions.casemgmtmid.details.table.item',
               value:api.'instructions_casemgmtmid_details_webService.txt']],
            [prepend:[
               token:'instructions.casemgmtmid.details.table.item',
               value:api.'instructions_casemgmtmid_details_DAVcontent.txt']],
         ]
      }
   }
   // ===================================================== TARGET: updateConfigure

   /**
      Target: updateConfigure 

      @see com.sas.gate.api.template.ConfigureTemplate
   */
   def updateConfigure()
   {
      //api.stop() 

      pass.with {
         t__debug = getProperty('wizard.debug.mode', '0') { toBoolean(it) }
      }

      String maintenanceFrom = getProperty('casemgmtmid.maintenance.from.version', '')

      if (maintenanceFrom)
      {
         String fromVersion         = api.baseVersion(maintenanceFrom)
         String maintenanceTo = getProperty('casemgmtmid.maintenance.to.version', '')
         String toVersion           = api.baseVersion(maintenanceTo)
         if (maintenanceFrom != maintenanceTo)
         {
            if (fromVersion != toVersion)
            {
               UpdateConfigureTemplate t = new UpdateConfigureTemplate(this)

               t['properties > pre']            .add(updateconfigure__pre_init)
               t['properties > init']           .add(properties_init)
               t['properties > post']           .add(properties_post)
               t['properties > post']           .add(updateconfigure__properties_post)
               t['properties > post']           .add(configure__init)
               t['properties > post']           .add(properties_assert).when('t__debug')
               t['loadmetadata > init']         .add(updateconfigure__loadmetadata_init)
               t['setobjectproperties > init']  .add(updateconfigure__setobjectproperties_init)
               t['docparts > init']             .add(updateconfigure__docparts_init)

               t.execute()
            }
            else
            {
               UpdateConfigureTemplate t = new UpdateConfigureTemplate(this)

               t['properties > init']        .add(properties_init)
               t['properties > post']        .add(properties_post)
               t['docparts > init']          .add(updateconfigure__docparts_l10n_init)

               t.execute()

            }
         }
      }
      else
         configure()

     //api.stop()
   }

   def updateconfigure__pre_init =
   {
      String legalName = api.'casemgmtmid.short.legal.name'

      String toVersion = api.baseVersion(api.'casemgmtmid.maintenance.to.version', api.'casemgmtmid.release.number')
      String fromVersion = api.baseVersion(api.'casemgmtmid.maintenance.from.version')
      api.createProperty('casemgmtmid.release.number', toVersion)
      String softwareComponent = "$legalName $toVersion"
      api.createProperty('casemgmtmid.software.component.name', softwareComponent)

      String applicationFolder = api.'casemgmtmid.application.folder'
      String oldFolderName = "$applicationFolder/$legalName $fromVersion"

      String configDir= "${api.'config.lev.web.applications.dir'}/${api.'casemgmtmid.normalized.offering.name'}"
      api.createProperty('casemgmtmid.config.dir', configDir)
      api.createProperty('casemgmtmid.configobj.version', toVersion)
      api.createProperty('casemgmtmid.display.name', softwareComponent)
      api.createProperty('casemgmtmid.configobj.name', softwareComponent)
      api.createProperty('casemgmtmid.app.name', softwareComponent)

      pass.with
      {

         _applicationSubFolder = api.createProperty('casemgmtmid.application.sub.folder', "$applicationFolder/$softwareComponent")

         t__removemetadatabypath =
         [
            paths:
            [
               [
                  objectPath     : oldFolderName,
                  deleteContents : true
               ],
            ]
         ]
      }
   }

   def updateconfigure__properties_post =
   {
      pass.with {

         t__createdirectories =
         [
            prefix: _configDir,
            entries:
            [
               'install/metadata',
            ]
         ]

      }
   }

   def updateconfigure__loadmetadata_init =
   {
      pass.with {
         t__loadmetadata_update =
         [
            file: canonical("$_configMetadataDir/casemgmtmid.appxml"),
            folder: _applicationSubFolder,
            applicationname: _softwareComponentName
         ]
      }
   }

   def updateconfigure__setobjectproperties_init =
   {
      pass.with {
         t__setobjectproperties +=
         [
            existingproperties: 'replace'
         ]
      }
   }

   /**
      Callback for hook: <hook>properties &gt; post</hook>

      @param t__docparts   <type>ArrayList&lt;Map&gt;</type>: tokens in Instructions.html to replace and properties to replace them with
            <indent><key>replace, prepend, append</key>- <type>String</type>: Type of replacement to perform<br>
            <indent><key>token</key>- <type>String</type>: Token in Instructions.html to replace<br>
            <key>value</key>- <type>String</type>: Property to replace token with</indent></indent>

   */
   def updateconfigure__docparts_init =
   {
      pass.with
      {
         t__docparts =
         [
            actions:
            [
               [replace:[
                  token:'instructions.casemgmtmid.head',
                  value:api.'instructions_casemgmtmid_head.txt']],
               [prepend:[
                  token:'instructions.casemgmtmid.details.table.head',
                  value:api.'instructions_casemgmtmid_details_table_head.txt']],
               [prepend:[
                  token:'instructions.casemgmtmid.details.table.item',
                  value:api.'updateconfig_casemgmtmid_details_table_item_upgrade.txt']],
               [prepend:[
                  token:'instructions.casemgmtmid.details.table.item',
                  value:api.'instructions_casemgmtmid_details_doc.txt']],
               [replace:[
                  token:'instructions.casemgmtmid.details.table.tail',
                  value:api.'instructions_casemgmtmid_details_table_tail.txt']],
            ]
         ]
      }
   }

   def updateconfigure__docparts_l10n_init =
   {
      pass.with
      {
         t__docparts =
         [
            actions:
            [
               [replace:[
                  token:'instructions.casemgmtmid.head',
                  value:api.'instructions_casemgmtmid_head.txt']],
               [prepend:[
                  token:'instructions.casemgmtmid.details.table.head',
                  value:api.'instructions_casemgmtmid_details_table_head.txt']],
               [prepend:[
                  token:'instructions.casemgmtmid.details.table.item',
                  value:api.'updateconfig_casemgmtmid_details_table_item_l10n.txt']],
               [replace:[
                  token:'instructions.casemgmtmid.details.table.tail',
                  value:api.'instructions_casemgmtmid_details_table_tail.txt']],
            ]
         ]
      }
   }

   // ===================================================== TARGET: checkForErrors

   /**
      Target: checkForErrors

      @see com.sas.gate.api.template.CheckForErrorsTemplate
   */
   def checkForErrors()
   {
      CheckForErrorsTemplate t = new CheckForErrorsTemplate(this)
      t.execute()
   }

   // ===================================================== TARGET: reportFailures

   /**
      Target: reportFailures

      @see com.sas.gate.api.template.ReportFailuresTemplate
   */
   def reportFailures()
   {
      ReportFailuresTemplate t = new ReportFailuresTemplate(this)
      t.execute()
   }

   // ===================================================== TARGET: unconfigure

   /**
      Target: unconfigure

      @see com.sas.gate.api.template.UnconfigureTemplate
   */
   def unconfigure()
   {
      //api.stop()

      UnconfigureTemplate t = new UnconfigureTemplate(this)

      t['properties > init']     .add(properties_init)
      t['properties > post']     .add(properties_post)
      t['init']                  .add(unconfigure__init)
      t['docparts > init']       .add(unconfigure__docparts_init)

      t.execute()

      //api.stop()
   }

   def unconfigure__init =
   {
      String applicationSubFolder   = api.'casemgmtmid.application.sub.folder'
      String appName                = "SAS$pass._normalizedShortLegalName$pass._releaseNumber"
      String datasourceName         = api.'casemgmtmid.dbms.datasource.name'
      String serverName             = api.serverName()

      pass.with
      {
         t__removemetadatabypath =
         [
            paths:
            [
               [
                  objectPath     : applicationSubFolder,
                  deleteContents : true
               ],
            ]
         ]

         t__removequeue =
         [
            queueName:              'AlertQueue',
            servername:             serverName,
         ]

         t__removeconnection =
         [
            connectionFactoryName:  'ECMQueueConnectionFactory',
            servername:             serverName,
         ]

         t__removedatasource =
         [
            datasourceName:   datasourceName,
            servername:       serverName
         ]

         t__undeployear =
         [
            undeploy:
            [
               [
                  earname: _rootAppName,
                  appname: appName
               ],
            ]
         ]
      }

   }
   /**
      Callback for hook: <hook>properties &gt; post</hook>

      @param t__docparts   <type>ArrayList&lt;Map&gt;</type>: tokens in ConfigurationRemoval.html to replace and properties to replace them with
            <indent><key>replace, prepend, append</key>- <type>String</type>: Type of replacement to perform<br>
            <indent><key>file</key>- <type>String</type>: File to replace tokens in.<br>
            <key>token</key>- <type>String</type>: Token in Instructions.html to replace<br>
            <key>value</key>- <type>String</type>: Property to replace token with</indent></indent>

   */
   def unconfigure__docparts_init =
   {
      pass.with
      {
         t__docparts =
         [
            actions:
            [
               [replace:[
                  file:    api.'config.lev.unconfiguration.instructions.file',
                  token:   'unconfiguration.summary.casemgmtmid.head',
                  value:   api.'unconfiguration_casemgmtmid_summary_head.txt']],
               [prepend:[
                  file:    api.'config.lev.unconfiguration.instructions.file',
                  token:   'unconfiguration.summary.casemgmtmid.item',
                  value:   api.'unconfiguration_casemgmtmid_summary_item_configobj.txt']],
               [prepend:[
                  file:    api.'config.lev.unconfiguration.instructions.file',
                  token:   'unconfiguration.summary.casemgmtmid.item',
                  value:   api.'unconfiguration_casemgmtmid_delete_ear.txt']],
               [replace:[
                  file:    api.'config.lev.unconfiguration.instructions.file',
                  token:   'unconfiguration.summary.casemgmtmid.tail',
                  value:   api.'unconfiguration_casemgmtmid_summary_tail.txt']],
            ]
         ]
      }
   }

   // ===================================================== TARGET: updatePasswords

   def updatePasswords()
   {
      //api.stop(properties:true)
      UpdatePasswordsTemplate t = new UpdatePasswordsTemplate(this)

      t['properties > init']        .add(properties_init)
      t['properties > post']        .add(properties_post)
      t['updatepasswords > init']   .add(updatepasswords__init)
      t['docparts > init']          .add(updatepasswords__docparts_init)

      t.execute()
      //api.stop(properties:true)
   }

   def updatepasswords__init =
   {
      String datasourceName                     = api.'casemgmtmid.dbms.datasource.name'
      String jndiName                           = "sas/jdbc/$datasourceName"
      String validationQuery =
         [
            db2:        'select 1 from sysibm.sysdummy1',
            oracle:     'select 1 from dual',
            postgres:   'select 1',
            sqlsvr:     'select 1 from sysobjects',
         ]."$pass._dbmsEngine"
      String serverName                         = api.serverName()
      String dbmsUrl =
         [
            db2:        "jdbc:db2://${api.'casemgmtmid.dbms.host'}:${api.'casemgmtmid.dbms.port'}/${api.'casemgmtmid.dbms.database.name'}",
            oracle:     "jdbc:oracle:thin:@//${api.'casemgmtmid.dbms.host'}:${api.'casemgmtmid.dbms.port'}/${api.'casemgmtmvac.dbms.oracle.net.service.name'}",
            postgres:   "jdbc:postgresql://${api.'casemgmtmid.dbms.host'}:${api.'casemgmtmid.dbms.port'}/${api.'casemgmtmid.dbms.database.name'}",
            sqlsvr:     "jdbc:sqlserver://${api.'casemgmtmid.dbms.host'}\\${api.'casemgmtmid.dbms.database.name'}:${api.'casemgmtmid.dbms.port'}",
         ]."$pass._dbmsEngine"

      pass.with {
         t__datasource_create =
         [
            adminUser:        api.'casemgmtmid.dbms.userid',
            adminPW:          api.decodeSASstring(api.'casemgmtmid.dbms.passwd'),
            classPath:        api.'casemgmtmid.dbms.oracle_driver',
            connectionURL:    dbmsUrl,
            datasourceName:   datasourceName,
            driver:           api.'casemgmtmvac.dbms.driver_class',
            jndiName:         jndiName,
            servername:       serverName,
            validationQuery:  validationQuery,
         ]

         t__datasource_connection =
         [
            connectionFactoryName:  'ECMQueueConnectionFactory',
            servername:             serverName,
         ]

         t__datasource_queue =
         [
            queueName:              'AlertQueue',
            servername:             serverName,
         ]
         t__updatepasswords =
         [
            // Corresponds to the "array" defined by <12byte>.credentials.persisted.list
            // and that for casemgmtmid is defined to be: casemgmtmid.dbms
            // with the implied suffixes: .userid and .passwd
            authdomains: [ api.'casemgmtmid.dbms.authdomain' ],
            createdatasource: t__datasource_create
         ]
      }
   }

   def updatepasswords__docparts_init =
   {
      pass.with
      {
         t__docparts =
         [
            file:    api.'config.lev.updatepasswords.file',
            actions:
            [
               [replace:[
                  token:'updatePasswords.summary.casemgmtmid.head',
                  value:api.'updatePasswords_casemgmtmid_summary_head.txt']],
               [prepend:[
                  token:'updatePasswords.summary.casemgmtmid.item',
                  value:api.'updatePasswords_casemgmtmid_summary_item.txt']],
               [replace:[
                  token:'updatePasswords.summary.casemgmtmid.tail',
                  value:api.'updatePasswords_casemgmtmid_summary_tail.txt']],
            ]
         ]
      }
   }

   // ===================================================== TARGET: updateContent

   def updateContent()
   {
      //api.stop()
      loadContent()
      //api.stop()
   }

   // ===================================================== TARGET: changeHost

   def changeHost()
   {
      //api.stop(properties:true)
      ChangeHostTemplate t = new ChangeHostTemplate(this)

      t['properties > init']  .add(properties_init)
      t['properties > post']  .add(properties_post)
      t['changehost > init']  .add(changehost__init)
      t['docparts > init']    .add(changehost__docparts_init)

      t.execute()
      //api.stop(properties:true)
   }

   def changehost__init =
   {
      pass.with {
         t__changehost =
         [
            exact: ['casemgmtmid.dbms.host':false],
         ]
      }
   }

   /**
      Callback for hook: <hook>docparts &gt; init</hook>

      @param t__docparts   <type>ArrayList&lt;Map&gt;</type>: tokens in Instructions.html to replace and properties to replace them with
            <indent><key>replace, prepend, append</key>- <type>String</type>: Type of replacement to perform<br>
            <indent><key>token</key>- <type>String</type>: Token in Instructions.html to replace<br>
            <key>value</key>- <type>String</type>: Property to replace token with</indent></indent>

   */
   def changehost__docparts_init =
   {
      pass.with
      {
         t__docparts =
         [
            file:    api.'config.lev.changehost.file',
            actions:
            [
               [prepend:[
                  token:   'changeHost.metadata.item',
                  value:   api.'changeHost_metadata_object_changed.txt']],
            ]
         ]
      }
   }
}
