/**
 * @class Comment
 * @description Represents discussion information regarding an entity. Comments with a parentId of 0 are considered
 *              top-level comments, and may contain replies, which are simply comments with a parentId that is not 0;
 *              these comments are treated as children to their parent.
 */
define([
    'dijit/focus',
    './DeleteCommentDialog',
    'dojox/html/entities',
    './Attachment',
    'dojox/lang/functional',
    '../CommentData',
    './../CustomDialog',
    'dijit/Dialog',
    '../AttachmentUploader/AttachmentUploader',
    './ReplyForm',
    'dojo/dom-style',
    'dojo/dom-construct',
    'dojo/on',
    'dojo/mouse',
    'dojo/dom-class',
    'dijit/_TemplatedMixin',
    'dijit/_WidgetBase',
    'dijit/_WidgetsInTemplateMixin',
    'dijit/form/Button',
    'dojo/_base/declare',
    'dojo/i18n!./nls/validate',
    'dojo/text!./templates/Comment.html',
    'fraudCommons/util/NLSUtil',
    'xstyle/css!./templates/Comment.css'
], function (focus, DeleteCommentDialog, entities, Attachment, functional, CommentData, CustomDialog, Dialog, AttachmentUploader, ReplyForm, domStyle, domConstruct, on, mouse, domClass, TemplatedMixin, WidgetBase, WidgetsInTemplateMixin, Button, declare, validate, template, NLSUtil) {

    return declare([WidgetBase, TemplatedMixin, WidgetsInTemplateMixin], {
        templateString: template,

        /**
         * @constructs Comment
         * @param {string} author - The author of the comment.
         * @param {string|Date} date - The date the comment was created.
         * @param {string} subject - The subject for the comment.
         * @param {number} commentId - The ID for the comment.
         * @param {string|number} objectId - The ID of the object to which this comment is linked.
         * @param {string|number} objectType - The type of the object to which this comment is linked.
         * @param {string} objectName - The name of the object to which this comment is linked.
         * @param {number} parentId - The ID of this comment's parent. A parentId of 0 means this comment is a parent.
         * @param {boolean} [editable=false] - True if this comment is editable by the current user.
         * @param {boolean} [deletable=false] - True if this comment is deletable by the current user.
         * @param {boolean} [attachmentsEditable=false] - True if this comment's attachments can be edited.
         * @param {array} attachments - The attachments for this object.
         * @param {function} onComplete - The callback to execute when this comment is updated.
         * @param {number} truncationLength - The comment length threshold after which text will be truncated and a
         *                                    "Show More" button will appear.
         */
        constructor: function (args) {
            this.author = args.author;
            this.date = args.date;
            this.categories = args.categories || [];
            this.subject = args.subject;
            this.commentId = args.commentId;
            this.objectId = args.objectId;
            this.objectType = args.objectType;
            this.objectName = args.objectName;
            this.parentId = args.parentId;
            this.editable = args.editable || false;
            this.deletable = args.deletable || false;
            this.attachmentsEditable = args.attachmentsEditable || false;
            this.attachments = args.attachments || [];
            this.previousText = "";
            this.onComplete = args.onComplete;
            this.truncationLength = args.truncationLength || 300;
            this.editButtonLabel = NLSUtil.resolve('btn_edit', validate);
            this.deleteButtonLabel = NLSUtil.resolve('btn_delete', validate);
            this.showMoreButtonLabel = NLSUtil.resolve('show_more', validate);
            this.showLessButtonLabel = NLSUtil.resolve('show_less', validate);
        },

        postCreate: function () {
            var that = this;
            var truncationLength = this.truncationLength;

            if (!this.editable || this.readOnly) {
                domStyle.set(this.editButton, {'display': 'none'});
            } else {
                domStyle.set(this.editButton, {'display': 'block'})
            }
            if (!this.deletable || this.readOnly) {
                domStyle.set(this.deleteButton, {'display': 'none'});
            } else {
                domStyle.set(this.deleteButton, {'display': 'block'})
            }

            /*
             * Turns the comment into a form for editing
             */
            on(this.editButton, "click", function () {
//                that.previousText = that.textContainer.get('content').split("<br>").join("\n");

                that.form = new ReplyForm({
                    categories: that.categories,
                    editForm: true,
                    commentId: that.commentId,
                    objectId: that.objectId,
                    objectName: that.objectName,
                    objectType: that.objectType,
                    subject: that.subject,
                    parentId: that.parentId,
                    newCategoriesAllowed: that.newCategoriesAllowed,
                    attachments: that.attachments,
                    emptyFileCallback: that.emptyFileCallback,
                    onComplete: that.onComplete,
                    cancelCallback: function () {
                        that._displayEditToolbar(true);
                        domClass.remove(that.domNode, 'editMode');
                    },
                    submitCallback: function () {
                        that.onComplete();
                        domClass.remove(that.domNode, 'editMode');
                    }
                });

                domClass.add(that.form.attachmentUploader.categorySelect.dropDown.domNode, 'categorySelectMenu');

                that._displayEditToolbar(false);

                domClass.add(that.domNode, 'editMode');

                that.form.textContainer.set('value', that.text);
                on(that.form.textContainer, "change", function () {
                    that.form.attachmentUploader.disableButtons(that.form.textContainer.textbox.value.trim().length <= 0);
                });
                domConstruct.place(that.form.domNode, that.attachmentsContainer, "before");
                that.form.textContainer.focus();
            });

            /*
             * Spawns a dialog that confirms that user wants to delete the comment. When the user confirms, the comment
             * is deleted.
             */
            on(this.deleteButton, "click", function () {
                var dialog = new CustomDialog({
                    title: NLSUtil.resolve('delete_confirm_title', validate),
                    confirmLabel: NLSUtil.resolve('btn_delete', validate),
                    cancelLabel: NLSUtil.resolve('cancel', validate),
                    body: new DeleteCommentDialog(),
                    confirmCallback: function () {
                        if (that.parentId === 0) {
                            CommentData.update({
                                commentId: that.commentId,
                                objectId: that.objectId,
                                objectName: that.objectName,
                                objectType: that.objectType,
                                parentId: that.parentId,
                                subject: that.subject,
                                pseudoDelete: true,
                                text: ""
                            }, that.onComplete);
                        } else {
                            CommentData.deleteComment(that.commentId, that.onComplete);
                        }
                        return true;
                    }
                });
                dialog.show();
            });

            /* Creates a list of attachments for the comment */
            functional.forEach(this.attachments, function (attachment, key) {
                var li = new Attachment({
                    title: attachment.name,
                    href: 'ViewAttachmentServlet?a_id=' + attachment.id
                });
                domConstruct.place(li.domNode, that.attachmentsContainer, "last");
            });

            /* Handle text truncation; handle expand/collapse */
            if (this.text && this.text.length > truncationLength) {
                // Truncate, then encode, then replace.
                var text = this.text;
                this.textContainer.set('content', entities.encode(this.text.substring(0, truncationLength)).replace(/\n/g, '<br />') + NLSUtil.resolve('ellipses', validate));
                domStyle.set(this.showMoreButton, 'display', 'inline-block');

                on(this.showMoreButton, 'click', function () {
                    that.textContainer.set('content', entities.encode(that.text).replace(/\n/g, '<br />'));
                    domStyle.set(this, 'display', 'none');
                    domStyle.set(that.showLessButton, 'display', 'inline-block');
                });

                on(this.showLessButton, 'click', function () {
                    that.textContainer.set('content', entities.encode(that.text.substring(0, truncationLength)).replace(/\n/g, '<br />') + NLSUtil.resolve('ellipses', validate));
                    domStyle.set(this, 'display', 'none');
                    domStyle.set(that.showMoreButton, 'display', 'inline-block');
                });

            } else {
                // Encode, then replace.
                this.textContainer.set('content', entities.encode(this.text).replace(/\n/g, '<br />'));
            }
        },

        /**
         * @private
         * @description Handles whether or not to display the toolbar with Edit and Delete buttons.
         * @param {boolean } bool - True if the toolbar should be shown.
         */
        _displayEditToolbar: function (bool) {
            var that = this;
            if (that.attachmentsContainer) {
                domStyle.set(that.attachmentsContainer, {
                    display: bool ? 'block' : 'none'
                });
            }
            if (that.textContainer) {
                domStyle.set(that.textContainer.domNode, {
                    display: bool ? 'block' : 'none'
                });
            }
            if (that.editToolbar) {
                domStyle.set(that.editToolbar.domNode, {
                    display: bool ? 'block' : 'none'
                });
            }
            if (that.collapseText) {
                domStyle.set(that.collapseText.domNode, {
                    display: bool ? 'block' : 'none'
                });
            }
        },

        resize: function () {
            this.inherited(arguments);
        },

        _setCategoriesAttr: function(/* Array */ c) {
            this._set('categories', c);
        },

        _getCategoriesAttr: function() {
            return this.categories;
        }

    });
});
